package cn.iocoder.yudao.module.member.service.scoreLog;

import cn.hutool.core.lang.generator.SnowflakeGenerator;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.controller.app.userScore.vo.AppMemberUserIdQueryVO;
import cn.iocoder.yudao.module.member.controller.app.userScore.vo.AppMemberUserScoreLogRespVO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreLog.MemberUserScoreLogDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserScoreLog.MemberUserScoreLogMapper;
import cn.iocoder.yudao.module.member.enums.ScoreOperateTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogBackVO;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogCreateReq;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogQueryVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.ScoreRuleBackDetailVO;
import cn.iocoder.yudao.module.reward.api.reward.RedeemRewardApi;
import cn.iocoder.yudao.module.reward.api.reward.dto.RedeemDetailForUserDTO;
import cn.iocoder.yudao.module.system.service.dict.DictTypeService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_ID_IS_NULL;

/**
 * 会员积分日志 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class MemberUserScoreLogServiceImpl extends AbstractService<MemberUserScoreLogMapper, MemberUserScoreLogDO> implements MemberUserScoreLogService {

    @Resource
    private MemberUserScoreLogMapper userScoreLogMapper;
    @Resource
    private SnowflakeGenerator snowflakeGenerator;
    @Resource
    private ScoreRuleService scoreRuleService;
    @Resource
    RedeemRewardApi redeemRewardApi;


    @Override
    public PageResult<MemberUserScoreLogBackVO> getPage(MemberUserScoreLogQueryVO query) {
        int start = (query.getPageNo() - 1) * query.getPageSize();
        int size = query.getPageSize();
        List<MemberUserScoreLogBackVO> list = userScoreLogMapper.getPageRecordList(start, size, query);
        int total = userScoreLogMapper.getPageCount(query);
        return new PageResult<>(list, total, query.getPageSize(), query.getPageNo(), (total + query.getPageSize() - 1) / query.getPageSize());
    }

    @Override
    public Long createScoreLog(MemberUserScoreLogCreateReq createReq) {
        if (createReq.getMemberId() == null) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        MemberUserScoreLogDO userScoreLogDO = new MemberUserScoreLogDO();
        userScoreLogDO.setId(snowflakeGenerator.next());
        userScoreLogDO.setMemberId(createReq.getMemberId());
        userScoreLogDO.setScoreCount(createReq.getScoreCount());
        userScoreLogDO.setOperateType(createReq.getOperateType());
        userScoreLogDO.setSourceType(createReq.getSourceType());
        userScoreLogDO.setRuleId(createReq.getRuleId());
        userScoreLogDO.setUniqueId(createReq.getUniqueId());
        if (createReq.getExtParam() != null) {
            userScoreLogDO.setExtParam(JSONUtil.toJsonStr(createReq.getExtParam()));
        }
        this.saveOrUpdate(userScoreLogDO);
        return userScoreLogDO.getId();
    }

    @Override
    public MemberUserScoreLogDO getByUniqueId(String uniqueId) {
        LambdaQueryWrapper<MemberUserScoreLogDO> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MemberUserScoreLogDO::getUniqueId, uniqueId);
        return this.selectOne(wrapper);
    }

    @Override
    public List<AppMemberUserScoreLogRespVO> getUserScoreLogList(AppMemberUserIdQueryVO query) {
        List<MemberUserScoreLogDO> userScoreLogDOS = userScoreLogMapper.selectList("member_id", query.getId());
        if (userScoreLogDOS == null || userScoreLogDOS.isEmpty()) {
            return Collections.EMPTY_LIST;
        }
        ArrayList<AppMemberUserScoreLogRespVO> appMemberUserScoreLogRespVOS = new ArrayList<>();
        for (MemberUserScoreLogDO userScoreLogDO : userScoreLogDOS) {
            setScoreRuleLogDetail(userScoreLogDO, appMemberUserScoreLogRespVOS);
        }
        return appMemberUserScoreLogRespVOS;
    }

    private void setScoreRuleLogDetail(MemberUserScoreLogDO userScoreLogDO, ArrayList<AppMemberUserScoreLogRespVO> appMemberUserScoreLogRespVOS) {
        AppMemberUserScoreLogRespVO appMemberUserScoreLogRespVO = new AppMemberUserScoreLogRespVO();
        appMemberUserScoreLogRespVO.setScoreCount(userScoreLogDO.getScoreCount());
        appMemberUserScoreLogRespVO.setOperateType(userScoreLogDO.getOperateType() == 1 ? "+" : "-");
        appMemberUserScoreLogRespVO.setCreateTime(userScoreLogDO.getCreateTime());
        // 增加积分日志
        if (Objects.equals(userScoreLogDO.getOperateType(), ScoreOperateTypeEnum.ADD.getValue())) {
            switch (userScoreLogDO.getSourceType()) {
                // 人工操作
                case 1:
                    appMemberUserScoreLogRespVO.setSourceTypeZh(DictFrameworkUtils.getDictDataFromCache("member_score_source", "1").getLabel());
                    appMemberUserScoreLogRespVO.setSourceTypeEn(DictFrameworkUtils.getDictDataFromCache("member_score_source", "1").getLabelEn());
                    appMemberUserScoreLogRespVO.setDescZh("人工积分");
                    appMemberUserScoreLogRespVO.setDescEn("Manually add points");
                    break;
                // 撤销兑换
                case 4:
                    appMemberUserScoreLogRespVO.setSourceTypeZh(DictFrameworkUtils.getDictDataFromCache("member_score_source", "4").getLabel());
                    appMemberUserScoreLogRespVO.setSourceTypeEn(DictFrameworkUtils.getDictDataFromCache("member_score_source", "4").getLabelEn());
                    // TODO：补充描述
                    appMemberUserScoreLogRespVO.setDescZh(DictFrameworkUtils.getDictDataFromCache("member_score_source", "4").getLabel());
                    appMemberUserScoreLogRespVO.setDescEn(DictFrameworkUtils.getDictDataFromCache("member_score_source", "4").getLabelEn());
                    break;
                // 活动积分
                case 5:
                case 6:
                case 7:
                case 8:
                    ScoreSourceTypeEnum scoreSourceTypeEnum = ScoreSourceTypeEnum.parseByValue(userScoreLogDO.getSourceType());
                    appMemberUserScoreLogRespVO.setSourceTypeZh(DictFrameworkUtils.getDictDataFromCache("member_score_source", String.valueOf(scoreSourceTypeEnum.getValue())).getLabel());
                    appMemberUserScoreLogRespVO.setSourceTypeEn(DictFrameworkUtils.getDictDataFromCache("member_score_source", String.valueOf(scoreSourceTypeEnum.getValue())).getLabelEn());
                    ScoreRuleBackDetailVO scoreRule = scoreRuleService.getScoreRule(userScoreLogDO.getRuleId());
                    if (scoreRule != null) {
                        appMemberUserScoreLogRespVO.setRuleId(userScoreLogDO.getRuleId());
                        appMemberUserScoreLogRespVO.setRuleNameEn(scoreRule.getTitleEn());
                        appMemberUserScoreLogRespVO.setRuleNameZh(scoreRule.getTitleZh());
                        appMemberUserScoreLogRespVO.setDescEn(scoreRule.getDescEn());
                        appMemberUserScoreLogRespVO.setDescZh(scoreRule.getDescZh());
                    }
                    break;
            }
        } else {
            // 减少积分日志
            switch (userScoreLogDO.getSourceType()) {
                // 人工操作
                case 1:
                    appMemberUserScoreLogRespVO.setSourceTypeZh(DictFrameworkUtils.getDictDataFromCache("member_score_source", "1").getLabel());
                    appMemberUserScoreLogRespVO.setSourceTypeEn(DictFrameworkUtils.getDictDataFromCache("member_score_source", "1").getLabelEn());
                    appMemberUserScoreLogRespVO.setDescZh("人工积分");
                    appMemberUserScoreLogRespVO.setDescEn("Manually reduce points");
                    break;
                // 兑换
                case 2:
                    appMemberUserScoreLogRespVO.setSourceTypeZh(DictFrameworkUtils.getDictDataFromCache("member_score_source", "2").getLabel());
                    appMemberUserScoreLogRespVO.setSourceTypeEn(DictFrameworkUtils.getDictDataFromCache("member_score_source", "2").getLabelEn());
                    // 查询兑换记录
                    Map<String, Long> extParam = JsonUtils.parseObject(userScoreLogDO.getExtParam(), HashMap.class);
                    Long redeemId = extParam.get("redeemId");
                    if (redeemId != null) {
                        RedeemDetailForUserDTO redeemDetailForUser = redeemRewardApi.getRedeemDetailForUser(redeemId);
                        if (redeemDetailForUser != null) {
                            appMemberUserScoreLogRespVO.setRewardCount(redeemDetailForUser.getRewardCount());
                            appMemberUserScoreLogRespVO.setDescZh(redeemDetailForUser.getRewardNameZh());
                            appMemberUserScoreLogRespVO.setDescEn(redeemDetailForUser.getRewardNameEn());
                        }
                    }
                    break;
                // 系统过期
                case 3:
                    appMemberUserScoreLogRespVO.setSourceTypeZh(DictFrameworkUtils.getDictDataFromCache("member_score_source", "3").getLabel());
                    appMemberUserScoreLogRespVO.setSourceTypeEn(DictFrameworkUtils.getDictDataFromCache("member_score_source", "3").getLabelEn());
                    appMemberUserScoreLogRespVO.setDescZh("系统过期");
                    appMemberUserScoreLogRespVO.setDescEn("System expiration");
            }
        }
        appMemberUserScoreLogRespVOS.add(appMemberUserScoreLogRespVO);
    }
}
